@@ -101,6 +101,16 @@ def _estimate_gaussian_mean(fn, order=_QUADRATURE_ORDER):
   return integral / np.sqrt(2*np.pi)
 
 
+def _estimate_2d_gaussian_mean(fn):
+  """Estimate the mean of a function fn(x, y) where x,y ~ N(0,1)."""
+
+  fn_weighted = lambda x, y: np.exp(-(x**2 + y**2) / 2) * fn(x, y)
+
+  integral, _ = sp_int.dblquad(fn_weighted, -10., 10., -10., 10., epsabs=0.0)
+
+  return integral / (2*np.pi)
+
+
 def _calc_c_map(activation, derivative_order=0, c=1.0, q_output=None):
   """Evaluate local C map value assuming an input q value of 1.
 
@@ -110,7 +120,7 @@ def _calc_c_map(activation, derivative_order=0, c=1.0, q_output=None):
     derivative_order: An integer giving the order of the derivative of the C map
       to take before evaluating it. (Default: 0)
     c: A float giving the input point at which to evaluate the C map. Must
-      be 0.0 or 1.0. (Default: 1.0)
+      be between -1.0 and 1.0. (Default: 1.0)
     q_output: Float or None giving the output q value associated with
       ``activation``, if this is known. If None this will be computed from
       scratch. (Default: None)
@@ -128,8 +138,12 @@ def _calc_c_map(activation, derivative_order=0, c=1.0, q_output=None):
     integral = _estimate_gaussian_mean(derivative)**2
   elif c == 1.0:
     integral = _estimate_gaussian_mean(lambda x: derivative(x)**2)
+  elif c >= -1.0 and c <= 1.0:
+    sqrt1mc2 = np.sqrt(1.0 - c**2)
+    integral = _estimate_2d_gaussian_mean(
+        lambda x, y: derivative(x) * derivative(c*x + sqrt1mc2*y))
   else:
-    raise NotImplementedError("Input c value must be 0.0 or 1.0.")
+    raise NotImplementedError("Input c value must be between -1.0 and 1.0.")
 
   if q_output is None:
     q_output = _estimate_gaussian_mean(lambda x: activation(x)**2)
@@ -190,16 +204,16 @@ def _transform_activation(phi, params):
 
     # Note: DO NOT use += and *= below! Bad things will happen.
     if input_scale is not None:
-      x = x * input_scale
+      x = x * float(input_scale)
     if input_shift is not None:
-      x = x + input_shift
+      x = x + float(input_shift)
 
     x = phi(x, **params)
 
     if output_shift is not None:
-      x = x + output_shift
+      x = x + float(output_shift)
     if output_scale is not None:
-      x = x * output_scale
+      x = x * float(output_scale)
 
     return x
 
@@ -246,7 +260,7 @@ def _solve_for_activation_params(
     opt_params = ("input_scale", "input_shift", "output_shift")
 
   def make_params_from_pvector(p):
-    """Make a dictionary of activation parameters value from a given vector."""
+    """Make a dictionary of activation parameters from a given vector."""
 
     params = constant_params.copy()
 
@@ -254,10 +268,11 @@ def _solve_for_activation_params(
       assert pname not in params
       params[pname] = p[i]
 
-    # Here we compute output_scale, and sometimes also output_shift, separately
-    # instead of taking them the vector.
+    # We directly compute output_scale, and sometimes also output_shift, instead
+    # of taking them from the parameter vector.
     output_shift, output_scale = _compute_output_params(
-        _transform_activation(_get_numpy_activation_function(name), params),
+        _transform_activation(
+            _get_numpy_activation_function(name), params),
         local_c_val_0_target)
 
     if output_shift is not None:
@@ -373,8 +388,15 @@ def _solve_increasing(fn, target, input_, min_, max_, tol=1e-8, max_eval=100):
         input_ = 0.5 * (input_ + max_)
 
   raise ValueError(f"Maximum evaluations ({max_eval}) exceeded while searching "
-                   "for solution. This is probably due the specified targets "
-                   "being unachievable for the given architecture.")
+                   "for solution. This is probably due the specified target "
+                   "being unachievable for the given architecture. For example,"
+                   " a Leaky-ReLU MLP of only a few layers may not be able "
+                   "to achieve the default C(0) target of 0.9 under TAT. "
+                   "Intuitively, this is because a shallow network cannot be "
+                   "made sufficiently nonlinear with such activation functions."
+                   " The solution to this would be to either use a smaller "
+                   "value for the C(0) target (corresponding to a more linear "
+                   "model), or to a use a deeper architecture.")
 
 
 def _compute_local_c_slope_1_target(max_slope_func, target_value):
@@ -470,7 +492,7 @@ def _get_activations_params(
   elif method == "TAT" and "leaky_relu" not in activation_names:
 
     if "relu" in activation_names:
-      raise ValueError("Standard ReLU not supported with TAT. Use leaky "
+      raise ValueError("Standard ReLU is not supported with TAT. Use leaky "
                        "ReLU instead.")
 
     c_curve_target = tat_params["c_curve_target"]
@@ -559,12 +581,18 @@ def _get_activations_params(
 def get_transformed_activations(
     activation_names, method="DKS", dks_params=None, tat_params=None,
     max_slope_func=None, max_curv_func=None, subnet_max_func=None,
-    activation_getter=_get_numpy_activation_function):
+    activation_getter=_get_numpy_activation_function,
+):
   """Gets transformed activation functions using the DKS or TAT method.
 
   See the DKS paper (https://arxiv.org/abs/2110.01765) and the TAT paper
   (https://openreview.net/forum?id=U0k7XNTiFEq) for details about what these
-  are, how they are computed, and what their parameters mean.
+  are, how they are computed, and what their parameters mean. A procedure to
+  compute the "maximal slope function" is given in the section titled "Summary
+  of our method" of the DKS paper. Procedures to compute the "maximal curvature
+  function", and the "subnet maximizing function", are given in the appendix
+  section titled "Additional details and pseudocode for activation function
+  transformations" of the TAT paper.
 
   Note that if you are using the JAX, PyTorch, or TensorFlow frameworks, you
   probably want to be using the version of get_transformed_activations() in the
@@ -581,7 +609,8 @@ def get_transformed_activations(
       getters in this package (for each framework) currently support the
       following names: "tanh", "softplus", "leaky_relu", "relu" (not
       recommended; use "leaky_relu" instead), "selu", "elu", "swish", "sigmoid",
-      "erf", "bentid", "atan", "asinh", "square", and "softsign".
+      "erf", "bentid", "atan", "asinh", "square", "softsign", "gelu", and
+      "gelu_exact".
     method: A string representing the method used to transform the activation
       functions. Can be "DKS", "TAT", or "untransformed". The latter choice
       will return activation functions without any transformations.
@@ -604,21 +633,22 @@ def get_transformed_activations(
       corresponds to "tau" from the paper), and defaults to 0.3. If
       ``tat_params`` is passed as None, it defaults to the empty dictionary (so
       that the parameters will use their default values). (Default: None)
-    max_slope_func: A callable which computes the maximal slope function, as
-      defined in the DKS paper. It should take a single argument representing
-      the slope of each local C map at ``c=1``. If this is required (i.e. when
-      using DKS) but is passed as None, it will be generated using
-      ``subnet_max_func`` if possible. (Default: None)
-    max_curv_func: A callable which computes the maximal curvature function. It
-      should take a single parameter representing the second derivative of each
-      local C map at c=1. If this is required (i.e. when using TAT with smooth
-      activation functions) but is passed as None, it will be generated using
-      ``subnet_max_func`` if possible. (Default: None)
-    subnet_max_func: A callable which computes the subnetwork maximizing
-      function of the network (denoted ``M_{f,r}(x)`` in the TAT paper). It
-      should take two arguments: the input value ``x``, and a callable
-      ``r_func`` which maps a float to a float. This is required when using TAT
-      with Leaky ReLUs. (Default: None)
+    max_slope_func: A callable which computes the "maximal slope function" of
+      the network, as defined in the DKS paper. It should take a single argument
+      representing the slope of each local C map at ``c=1``. If this is required
+      (i.e. when using DKS) but passed as None, it will be generated using
+      ``subnet_max_func`` when possible. (Default: None)
+    max_curv_func: A callable which computes the "maximal curvature function" of
+      the network, as defined in the TAT paper. It should take a single
+      parameter representing the second derivative of each local C map at c=1.
+      If this is required (i.e. when using TAT with smooth activation functions)
+      but is passed as None, it will be generated using ``subnet_max_func`` when
+      possible. (Default: None)
+    subnet_max_func: A callable which computes the "subnetwork maximizing
+      function" of the network, as defined in the TAT paper (and denoted
+      ``M_{f,r}(x)``). It should take two arguments: the input value ``x``, and
+      a callable ``r_fn`` which maps a float to a float. This is required when
+      using TAT with Leaky ReLUs. (Default: None)
     activation_getter: A callable which takes a string name for an activation
       function and returns the (untransformed) activation function corresponding
       to this name. Defaults to one returning activation functions in NumPy
